@@ -47,14 +47,16 @@ my $account = $rh->accounts()->{results}[0];   # Accounts are a paginated list
 my $instrument = $rh->instrument($symbol);     # Find the instrument we want
 my $order =
     Finance::Robinhood::Order->new(
-                     account    => $account,
-                     instrument => $instrument,
-                     type       => 'market',
-                     price => $instrument->quote()->{results}[0]->bid_price(),
-                     trigger       => 'immediate',
-                     time_in_force => 'gfd',
-                     side          => 'buy',
-                     quantity      => $quantity
+    account    => $account,
+    instrument => $instrument,
+    type       => 'limit',
+    price      => $instrument->last_extended_hours_trade_price
+        // $instrument->last_trade_price // $instrument->quote()->ask_price(),
+    trigger        => 'immediate',
+    time_in_force  => 'gfd',
+    side           => 'buy',
+    quantity       => $quantity,
+    extended_hours => 1
     );
 $order
     && printf
