@@ -3,7 +3,7 @@ use 5.010;
 use strict;
 use warnings;
 use Carp;
-our $VERSION = "0.19";
+our $VERSION = "0.20";
 use Moo;
 use JSON::Tiny qw[decode_json];
 use strictures 2;
@@ -22,7 +22,7 @@ has $_ => (
     )
     for (
         qw[bloomberg_unique day_trade_ratio min_tick_size margin_initial_ratio
-        id maintenance_ratio name state symbol tradeable country]);
+        id maintenance_ratio name state symbol tradeable country type]);
 #
 has $_ => (is     => 'ro',
            coerce => \&Finance::Robinhood::_2_datetime
@@ -59,7 +59,7 @@ sub _build_quote {
     return Finance::Robinhood::quote(shift->symbol())->{results}[0];
 }
 
-sub _build_historicals {
+sub historicals {
     return Finance::Robinhood::historicals(shift->symbol(), shift, shift);
 }
 
@@ -134,6 +134,11 @@ C<year>, or C<5year>.
 
 This makes an API call for information this particular instrument is traded on.
 
+=head2 C<type( )>
+
+What sort of instrument this is. May be a C<stock>, C<adr>, C<cef>, C<reit>,
+C<etp>, etc.
+
 =head2 C<tradeable( )>
 
 Returns a boolean value indicating whether this security can be traded on
